@@ -11,10 +11,13 @@ pub trait Repo {
     fn head_commit(&self) -> std::rc::Rc<Commit>;
     fn head_branch(&self) -> Option<Branch>;
     fn resolve(&self, revspec: &str) -> Option<std::rc::Rc<Commit>>;
-    fn commits_from(
+    fn parent_ids(&self, head_id: git2::Oid) -> Result<Vec<git2::Oid>, git2::Error>;
+    fn commit_count(&self, base_id: git2::Oid, head_id: git2::Oid) -> Option<usize>;
+    fn commit_range(
         &self,
-        head_id: git2::Oid,
-    ) -> Box<dyn Iterator<Item = std::rc::Rc<Commit>> + '_>;
+        base_bound: std::ops::Bound<&git2::Oid>,
+        head_bound: std::ops::Bound<&git2::Oid>,
+    ) -> Result<Vec<git2::Oid>, git2::Error>;
     fn contains_commit(
         &self,
         haystack_id: git2::Oid,
@@ -165,6 +168,10 @@ impl GitRepo {
     }
 
     pub fn merge_base(&self, one: git2::Oid, two: git2::Oid) -> Option<git2::Oid> {
+        if one == two {
+            return Some(one);
+        }
+
         self.repo.merge_base(one, two).ok()
     }
 
@@ -240,17 +247,64 @@ impl GitRepo {
         self.find_commit(id)
     }
 
-    pub fn commits_from(
-        &self,
-        head_id: git2::Oid,
-    ) -> impl Iterator<Item = std::rc::Rc<Commit>> + '_ {
+    pub fn parent_ids(&self, head_id: git2::Oid) -> Result<Vec<git2::Oid>, git2::Error> {
+        let commit = self.repo.find_commit(head_id)?;
+        Ok(commit.parent_ids().collect())
+    }
+
+    pub fn commit_count(&self, base_id: git2::Oid, head_id: git2::Oid) -> Option<usize> {
+        if base_id == head_id {
+            return Some(0);
+        }
+
+        let merge_base_id = self.merge_base(base_id, head_id)?;
+        if merge_base_id != base_id {
+            return None;
+        }
         let mut revwalk = self.repo.revwalk().unwrap();
         revwalk.push(head_id).unwrap();
-        revwalk.set_sorting(git2::Sort::TOPOLOGICAL).unwrap();
+        revwalk.hide(base_id).unwrap();
+        Some(revwalk.count())
+    }
 
-        revwalk
+    pub fn commit_range(
+        &self,
+        base_bound: std::ops::Bound<&git2::Oid>,
+        head_bound: std::ops::Bound<&git2::Oid>,
+    ) -> Result<Vec<git2::Oid>, git2::Error> {
+        let head_id = match head_bound {
+            std::ops::Bound::Included(head_id) | std::ops::Bound::Excluded(head_id) => *head_id,
+            std::ops::Bound::Unbounded => panic!("commit_range's HEAD cannot be unbounded"),
+        };
+        let skip = if matches!(head_bound, std::ops::Bound::Included(_)) {
+            0
+        } else {
+            1
+        };
+
+        let base_id = match base_bound {
+            std::ops::Bound::Included(base_id) | std::ops::Bound::Excluded(base_id) => {
+                debug_assert_eq!(self.merge_base(*base_id, head_id), Some(*base_id));
+                Some(*base_id)
+            }
+            std::ops::Bound::Unbounded => None,
+        };
+
+        let mut revwalk = self.repo.revwalk()?;
+        revwalk.push(head_id)?;
+        if let Some(base_id) = base_id {
+            revwalk.hide(base_id)?;
+        }
+        revwalk.set_sorting(git2::Sort::TOPOLOGICAL)?;
+        let mut result = revwalk
             .filter_map(Result::ok)
-            .filter_map(move |oid| self.find_commit(oid))
+            .skip(skip)
+            .take_while(|id| Some(*id) != base_id)
+            .collect::<Vec<_>>();
+        if let std::ops::Bound::Included(base_id) = base_bound {
+            result.push(*base_id);
+        }
+        Ok(result)
     }
 
     pub fn contains_commit(
@@ -314,79 +368,7 @@ impl GitRepo {
         head_id: git2::Oid,
         cherry_id: git2::Oid,
     ) -> Result<git2::Oid, git2::Error> {
-        let base_id = self
-            .commits_from(cherry_id)
-            .filter(|c| c.id != cherry_id)
-            .map(|c| c.id)
-            .next()
-            .unwrap_or(cherry_id);
-        if base_id == head_id {
-            return Ok(cherry_id);
-        }
-        let base_ann_commit = self.repo.find_annotated_commit(base_id)?;
-        let head_ann_commit = self.repo.find_annotated_commit(head_id)?;
-        let cherry_ann_commit = self.repo.find_annotated_commit(cherry_id)?;
-        let cherry_commit = self.repo.find_commit(cherry_id)?;
-        let mut rebase = self.repo.rebase(
-            Some(&cherry_ann_commit),
-            Some(&base_ann_commit),
-            Some(&head_ann_commit),
-            Some(git2::RebaseOptions::new().inmemory(true)),
-        )?;
-
-        let mut tip_id = head_id;
-        while let Some(op) = rebase.next() {
-            op.map_err(|e| {
-                let _ = rebase.abort();
-                e
-            })?;
-            let inmemory_index = rebase.inmemory_index().unwrap();
-            if inmemory_index.has_conflicts() {
-                let conflicts = inmemory_index
-                    .conflicts()?
-                    .map(|conflict| {
-                        let conflict = conflict.unwrap();
-                        let our_path = conflict
-                            .our
-                            .as_ref()
-                            .map(|c| bytes2path(&c.path))
-                            .or_else(|| conflict.their.as_ref().map(|c| bytes2path(&c.path)))
-                            .or_else(|| conflict.ancestor.as_ref().map(|c| bytes2path(&c.path)))
-                            .unwrap_or_else(|| std::path::Path::new("<unknown>"));
-                        format!("{}", our_path.display())
-                    })
-                    .join("\n  ");
-                return Err(git2::Error::new(
-                    git2::ErrorCode::Unmerged,
-                    git2::ErrorClass::Index,
-                    format!("cherry-pick conflicts:\n  {}\n", conflicts),
-                ));
-            }
-
-            let mut sig = self.repo.signature()?;
-            if let (Some(name), Some(email)) = (sig.name(), sig.email()) {
-                // For simple rebases, preserve the original commit time
-                sig = git2::Signature::new(name, email, &cherry_commit.time())?.to_owned();
-            }
-            let commit_id = match rebase.commit(None, &sig, None).map_err(|e| {
-                let _ = rebase.abort();
-                e
-            }) {
-                Ok(commit_id) => Ok(commit_id),
-                Err(err) => {
-                    if err.class() == git2::ErrorClass::Rebase
-                        && err.code() == git2::ErrorCode::Applied
-                    {
-                        log::trace!("Skipping {}, already applied to {}", cherry_id, head_id);
-                        return Ok(tip_id);
-                    }
-                    Err(err)
-                }
-            }?;
-            tip_id = commit_id;
-        }
-        rebase.finish(None)?;
-        Ok(tip_id)
+        git2_ext::ops::cherry_pick(&self.repo, head_id, cherry_id)
     }
 
     pub fn squash(
@@ -394,65 +376,7 @@ impl GitRepo {
         head_id: git2::Oid,
         into_id: git2::Oid,
     ) -> Result<git2::Oid, git2::Error> {
-        // Based on https://www.pygit2.org/recipes/git-cherry-pick.html
-        let head_commit = self.repo.find_commit(head_id)?;
-        let head_tree = self.repo.find_tree(head_commit.tree_id())?;
-
-        let base_commit = if 0 < head_commit.parent_count() {
-            head_commit.parent(0)?
-        } else {
-            head_commit.clone()
-        };
-        let base_tree = self.repo.find_tree(base_commit.tree_id())?;
-
-        let into_commit = self.repo.find_commit(into_id)?;
-        let into_tree = self.repo.find_tree(into_commit.tree_id())?;
-
-        let onto_commit;
-        let onto_commits;
-        let onto_commits: &[&git2::Commit] = if 0 < into_commit.parent_count() {
-            onto_commit = into_commit.parent(0)?;
-            onto_commits = [&onto_commit];
-            &onto_commits
-        } else {
-            &[]
-        };
-
-        let mut result_index = self
-            .repo
-            .merge_trees(&base_tree, &into_tree, &head_tree, None)?;
-        if result_index.has_conflicts() {
-            let conflicts = result_index
-                .conflicts()?
-                .map(|conflict| {
-                    let conflict = conflict.unwrap();
-                    let our_path = conflict
-                        .our
-                        .as_ref()
-                        .map(|c| bytes2path(&c.path))
-                        .or_else(|| conflict.their.as_ref().map(|c| bytes2path(&c.path)))
-                        .or_else(|| conflict.ancestor.as_ref().map(|c| bytes2path(&c.path)))
-                        .unwrap_or_else(|| std::path::Path::new("<unknown>"));
-                    format!("{}", our_path.display())
-                })
-                .join("\n  ");
-            return Err(git2::Error::new(
-                git2::ErrorCode::Unmerged,
-                git2::ErrorClass::Index,
-                format!("squash conflicts:\n  {}\n", conflicts),
-            ));
-        }
-        let result_id = result_index.write_tree_to(&self.repo)?;
-        let result_tree = self.repo.find_tree(result_id)?;
-        let new_id = self.repo.commit(
-            None,
-            &into_commit.author(),
-            &into_commit.committer(),
-            into_commit.message().unwrap(),
-            &result_tree,
-            onto_commits,
-        )?;
-        Ok(new_id)
+        git2_ext::ops::squash(&self.repo, head_id, into_id)
     }
 
     pub fn stash_push(&mut self, message: Option<&str>) -> Result<git2::Oid, git2::Error> {
@@ -640,11 +564,20 @@ impl Repo for GitRepo {
         self.resolve(revspec)
     }
 
-    fn commits_from(
+    fn parent_ids(&self, head_id: git2::Oid) -> Result<Vec<git2::Oid>, git2::Error> {
+        self.parent_ids(head_id)
+    }
+
+    fn commit_count(&self, base_id: git2::Oid, head_id: git2::Oid) -> Option<usize> {
+        self.commit_count(base_id, head_id)
+    }
+
+    fn commit_range(
         &self,
-        head_id: git2::Oid,
-    ) -> Box<dyn Iterator<Item = std::rc::Rc<Commit>> + '_> {
-        Box::new(self.commits_from(head_id))
+        base_bound: std::ops::Bound<&git2::Oid>,
+        head_bound: std::ops::Bound<&git2::Oid>,
+    ) -> Result<Vec<git2::Oid>, git2::Error> {
+        self.commit_range(base_bound, head_bound)
     }
 
     fn contains_commit(
@@ -790,10 +723,15 @@ impl InMemoryRepo {
         self.find_commit(branch.id)
     }
 
-    pub fn commits_from(
-        &self,
-        head_id: git2::Oid,
-    ) -> impl Iterator<Item = std::rc::Rc<Commit>> + '_ {
+    pub fn parent_ids(&self, head_id: git2::Oid) -> Result<Vec<git2::Oid>, git2::Error> {
+        let next = self
+            .commits
+            .get(&head_id)
+            .and_then(|(parent, _commit)| *parent);
+        Ok(next.into_iter().collect())
+    }
+
+    fn commits_from(&self, head_id: git2::Oid) -> impl Iterator<Item = std::rc::Rc<Commit>> + '_ {
         let next = self.commits.get(&head_id).cloned();
         CommitsFrom {
             commits: &self.commits,
@@ -801,6 +739,50 @@ impl InMemoryRepo {
         }
     }
 
+    pub fn commit_count(&self, base_id: git2::Oid, head_id: git2::Oid) -> Option<usize> {
+        let merge_base_id = self.merge_base(base_id, head_id)?;
+        let count = self
+            .commits_from(head_id)
+            .take_while(move |cur_id| cur_id.id != merge_base_id)
+            .count();
+        Some(count)
+    }
+
+    pub fn commit_range(
+        &self,
+        base_bound: std::ops::Bound<&git2::Oid>,
+        head_bound: std::ops::Bound<&git2::Oid>,
+    ) -> Result<Vec<git2::Oid>, git2::Error> {
+        let head_id = match head_bound {
+            std::ops::Bound::Included(head_id) | std::ops::Bound::Excluded(head_id) => *head_id,
+            std::ops::Bound::Unbounded => panic!("commit_range's HEAD cannot be unbounded"),
+        };
+        let skip = if matches!(head_bound, std::ops::Bound::Included(_)) {
+            0
+        } else {
+            1
+        };
+
+        let base_id = match base_bound {
+            std::ops::Bound::Included(base_id) | std::ops::Bound::Excluded(base_id) => {
+                debug_assert_eq!(self.merge_base(*base_id, head_id), Some(*base_id));
+                Some(*base_id)
+            }
+            std::ops::Bound::Unbounded => None,
+        };
+
+        let mut result = self
+            .commits_from(head_id)
+            .skip(skip)
+            .map(|commit| commit.id)
+            .take_while(|id| Some(*id) != base_id)
+            .collect::<Vec<_>>();
+        if let std::ops::Bound::Included(base_id) = base_bound {
+            result.push(*base_id);
+        }
+        Ok(result)
+    }
+
     pub fn contains_commit(
         &self,
         haystack_id: git2::Oid,
@@ -982,11 +964,20 @@ impl Repo for InMemoryRepo {
         self.resolve(revspec)
     }
 
-    fn commits_from(
+    fn parent_ids(&self, head_id: git2::Oid) -> Result<Vec<git2::Oid>, git2::Error> {
+        self.parent_ids(head_id)
+    }
+
+    fn commit_count(&self, base_id: git2::Oid, head_id: git2::Oid) -> Option<usize> {
+        self.commit_count(base_id, head_id)
+    }
+
+    fn commit_range(
         &self,
-        head_id: git2::Oid,
-    ) -> Box<dyn Iterator<Item = std::rc::Rc<Commit>> + '_> {
-        Box::new(self.commits_from(head_id))
+        base_bound: std::ops::Bound<&git2::Oid>,
+        head_bound: std::ops::Bound<&git2::Oid>,
+    ) -> Result<Vec<git2::Oid>, git2::Error> {
+        self.commit_range(base_bound, head_bound)
     }
 
     fn contains_commit(
@@ -1046,20 +1037,6 @@ impl Repo for InMemoryRepo {
     }
 }
 
-// From git2 crate
-#[cfg(unix)]
-fn bytes2path(b: &[u8]) -> &std::path::Path {
-    use std::os::unix::prelude::*;
-    std::path::Path::new(std::ffi::OsStr::from_bytes(b))
-}
-
-// From git2 crate
-#[cfg(windows)]
-fn bytes2path(b: &[u8]) -> &std::path::Path {
-    use std::str;
-    std::path::Path::new(str::from_utf8(b).unwrap())
-}
-
 pub fn stash_push(repo: &mut dyn Repo, context: &str) -> Option<git2::Oid> {
     let branch = repo.head_branch();
     let stash_msg = format!(
@@ -1095,3 +1072,12 @@ pub fn stash_pop(repo: &mut dyn Repo, stash_id: Option<git2::Oid>) {
         }
     }
 }
+
+pub fn commit_range(
+    repo: &dyn Repo,
+    head_to_base: impl std::ops::RangeBounds<git2::Oid>,
+) -> Result<Vec<git2::Oid>, git2::Error> {
+    let head_bound = head_to_base.start_bound();
+    let base_bound = head_to_base.end_bound();
+    repo.commit_range(base_bound, head_bound)
+}
