@@ -3,5 +3,3 @@
 pub mod config;
 pub mod git;
 pub mod graph;
-pub mod log;
-pub mod stash;
